#!/usr/bin/env python
from itertools import count, izip, combinations

import utils
from const import HIGH_PITCH_LIMIT, LOW_PITCH_LIMIT, Z_PARTNERS


class SetRowBase(object):
    """Base class for PC/pitch sets and tone rows"""

    #TODO: limit pitches to within PITCH_LIMIT bounds
    pitches = []
    _mod = 12
    _default_m = 5
    _ordered = False
    _multiset = False

    #Overrides
    def __init__(self, *args, **kwargs):
        modulus = kwargs.pop('mod', 12)
        self.mod(modulus)
        ordered = kwargs.pop('ordered', self._ordered)
        self.ordered(ordered)
        multiset = kwargs.pop('multiset', self._multiset)
        self.multiset(multiset)
        self.pitches = []
        if len(args) == 1:
            self.pitches = self.__add__(args[0], internal=True)
        elif len(args) > 1:
            result = []
            for arg in args:
                result += self.__add__(arg, internal=True)
            self.pitches = result
        if not self._multiset:
            self.pitches = self._rm_dupes(self.pitches)

    def __add__(self, other, **kwargs):
        internal = kwargs.get('internal', False)
        ps = other
        if isinstance(other, SetRowBase):
            ps = other.pitches[:]
        if isinstance(other, (int, long)):
            ps = [other]
        if isinstance(other, (tuple, list)):
            ps = list(other)
        if isinstance(other, set):
            ps = [int(num) for num in other]
        ps = self.pitches[:] + ps
        if not self._multiset:
            ps = self._rm_dupes(ps)
        # If initializing, returning an instance creates infinite recursion
        ps = self.copy(ps) if not internal else ps
        return ps

    def __radd__(self, other):
        return self.__add__(other)

    def __eq__(self, other):
        """Compare equality between a ToneRow/PSet/PCSet and another object"""
        try:
            that = other.copy()
        except AttributeError:
            try:
                that = other[:]
            except TypeError:
                that = [other]
        #Are we comparing pitches as ordered lists or set membership?
        if not self._ordered and isinstance(other, (tuple, list)):
            that = set(that)
        #Is the other a PSet/PCSet or builtin ?
        if isinstance(that, SetRowBase):
            return self.ppc == that.ppc
        if isinstance(that, (tuple, list)):
            return self.ppc == list(that)
        if isinstance(that, set):
            return set(self.ppc) == that
        else:
            return NotImplemented
        return False

    def __ne__(self, other):
        result = self.__eq__(other)
        if result is NotImplemented:
            return result
        return not result

    def __getitem__(self, key):
        try:
            l = []
            l.extend(self.ppc[key])
            return self.copy(l)
        except TypeError:
            l = [self.ppc[key]]
            return self.copy(l)

    def __setitem__(self, key, value):
        self.pitches[key] = value

    def __iter__(self):
        for p in self.ppc:
            yield p

    def __len__(self):
        return len(self.ppc)

    def __repr__(self):
        return str(self.ppc)

    def copy(self, pitches=None):
        """Use to copy a ToneRow/PSet/PCSet with all data attributes."""
        if pitches is None:
            pitches = self.pitches
        new = self.__class__(pitches, mod=self._mod, ordered=self._ordered,
                             multiset=self._multiset)
        if isinstance(self, PPCSetBase):
            new.canon(self._canon_t, self._canon_i, self._canon_m)
        new._default_m = self._default_m
        return new

    def mod(self, new_mod=None):
        """
        Takes one argument as the new modulus of the system.
        Without an argument, returns the current modulus.
        """
        if new_mod:
            self._mod = new_mod
        else:
            return self._mod

    def default_m(self, new_m=None):
        """
        Takes one argument as the new default argument for M operations.
        (The default for Mod 12 is 5)
        Without an argument, returns the current default m.
        """
        if new_m:
            self._default_m = new_m
        else:
            return self._default_m

    def multiset(self, value=None):
        """
        Takes one boolean argument and determines if the object is a multiset.
        (The default for all objects is False. ToneRows cannot be multisets)
        Without an argument, returns the current setting.
        """
        if value is not None:        
            self._multiset = True if value else False
        else:
            return self._multiset

    def ordered(self, value=None):
        """
        Takes one boolean argument and determines if the object is ordered.
        (The default for PCSets is False. The default for PSets is True.)
        Without an argument, returns the current setting.
        """
        if value is not None:
            self._ordered = True if value else False
        else:
            return self._ordered

    @property
    def pcs(self):
        """Returns the pitch classes of the current set/row"""
        return [pitch % self._mod for pitch in self.pitches]

    @property
    def _pc_set(self):
        """Returns pitch classes as a Python set for internal use"""
        return set(self.pcs)

    @property
    def _pitch_set(self):
        """Returns pitches as a Python set for internal use"""
        return set(self.pitches)

    @property
    def uo_pcs(self):
        """Returns unordered pitch classes in ascending order"""
        output = self.pcs[:]
        output.sort()
        return output

    @property
    def _unique_pcs(self):
        """
        Returns the unique, unordered pitch classes in ascending order. These
        are guarnteed to be unique regardless of rather or not the object is a
        multiset.
        """
        output = list(self._pc_set)
        output.sort()
        return output

    @property
    def uo_pitches(self):
        """Returns the unordered pitches in ascending order"""
        output = self.pitches[:]
        output.sort()
        return output

    @property
    def ppc(self):
        """
        Returns the pitches or pcs of a ToneRow, PCSet, or PSet taking into
        account the ordered and multiset settings.
        """
        #Ordered?
        if self._ordered:
            pitches = self.pitches
            pcs = self.pcs
        else:
            pitches = self.uo_pitches
            pcs = self.uo_pcs
        #PC/Pitch Set?
        if isinstance(self, PSet):
            ppc = pitches
        else:
            ppc = pcs
        #Multiset?
        if not self._multiset:
            ppc = self._rm_dupes(ppc)
        return ppc

    def _rm_dupes(self, ps):
        """Remove all duplicates of a given pitch or pitch class from a list"""
        new = []
        [new.insert(ps.index(num), num) for num in ps[:] if num not in new]
        return new

    def each_n(self):
        """
        Yields a number for each possible member in the object considering its
        modulus.
        (An object with a modulus of 12 would return [0, 1, 2...11])
        """
        return self.each_n_in_mod(self._mod)

    @classmethod
    def each_n_in_mod(cls, mod):
        """
        Same as the instance method but takes one positional arg as the modulus
        """
        for num in xrange(0, mod):
            yield num

    def each_tto(self):
        """
        Yields an (n, m) pair for each TTO that can be performed on the given
        object
        """
        for m in [1, -1, self._default_m, self._mod - self._default_m]:
            for n in self.each_n():
                yield (n, m)

    def _transpose(self, sub_n=0):
        return self.copy(utils.transpose(self.pitches, sub_n))

    def _invert(self, sub_n=0):
        return self.copy(utils.invert(self.pitches, sub_n))

    def _transpose_multiply(self, sub_n=0, sub_m=0):
        if not sub_m:
            sub_m = self._default_m
        result = [pc % self._mod for pc in \
                  utils.transpose_multiply(self.pcs, sub_n, sub_m)]
        return self.copy(result)

    def t(self, sub_n):
        """Transpose the object in place by the argument provided."""
        self[:] = (self._transpose(sub_n)).pitches

    def i(self, sub_n=0):
        """
        Invert the object in place. If an argument is provided, also transpose
        the object in place by that amount.
        """
        self[:] = (self._invert(sub_n)).pitches

    @property
    def t_rotations(self):
        """
        Returns a list of objects for each possible transposition of the given
        object.
        """
        return [self.copy(rot) for rot in [self._transpose(n) \
                                           for n in self.each_n()]]

    @property
    def i_rotations(self):
        """
        Returns a list of objects for each possible transposition of the given
        object after inversion.
        """
        return [self.copy(rot) for rot in [self._invert(n) \
                                           for n in self.each_n()]]

    @property
    def m_rotations(self):
        """
        Returns a list of objects for each possible transposition of the given
        object after M.
        """
        return [self.copy(rot) for rot in [self._transpose_multiply(n) \
                                           for n in self.each_n()]]

    @property
    def mi_rotations(self):
        """
        Returns a list of objects for each possible transposition of the given
        object after MI.
        """
        return [self.copy(rot) for rot in [self._transpose_multiply(n, self._default_m * -1) \
                                           for n in self.each_n()]]

    @property
    def all_rotations(self):
        """
        Return a flat list of objects for each possible TTO of the given object
        """
        return [self._transpose_multiply(n, m) for n, m in self.each_tto()]


class PCBase(object):
    """Base class for Tone rows and PC sets"""

    def m(self, sub_n=0):
        """
        Perform M on the object in place. If an argument is provided, also
        transpose the object in place by that amount.
        """
        self[:] = (self._transpose_multiply(sub_n, self._default_m)).pitches

    def mi(self, sub_n=0):
        """
        Perform M and I on the object in place. If an argument is provided,
        also transpose the object in play by that amount.
        """
        sub_m = self._mod - self._default_m
        self[:] = (self._transpose_multiply(sub_n, sub_m)).pitches
        
    def t_m(self, sub_n, sub_m):
        """
        Perform TnMm on the object in place, where n and m are positional
        arguments. If n is not provided, it defaults to 0. If m is not provided
        it defaults to the default_m of the object.
        """
        self[:] = (self._transpose_multiply(sub_n, sub_m)).pitches


class ToneRow(SetRowBase, PCBase):
    _modulus = 12

    class IncompleteToneRow(Exception):
        pass

    def __init__(self, *args, **kwargs):
        kwargs.update({'multiset': False, 'ordered': True})
        SetRowBase.__init__(self, *args, **kwargs)
        if len(self.ppc) < self._mod:
            msg = 'Tone rows must be instantiated with a number of ' + \
                  'pitches or pcs equal to their modulus'
            raise self.IncompleteToneRow(msg)

    def swap(self, a, b):
        """
        Given two arguments, swap the PC's in the ToneRow that are at these
        positions. 
        """
        c = self.pitches[a]
        self.pitches[a] = self.pitches[b]
        self.pitches[b] = c

    @property
    def P(self):
        """Returns the prime of the ToneRow"""
        return self

    @property
    def R(self):
        """Returns the retrograde of the ToneRow"""
        ppc = self.ppc
        ppc.reverse()
        return self.copy(ppc)

    @property
    def I(self):
        """Returns the inversion of the ToneRow"""
        return self._invert()

    @property
    def RI(self):
        """Returns the retrograde inversion of the ToneRow"""
        return self.R.I

    @property
    def M(self):
        """
        Returns the Mm of the ToneRow, where m = the default_m of the ToneRow
        """
        return self._transpose_multiply()

    @property
    def MI(self):
        """
        Returns the MmI of the ToneRow, where m = the default_m of the ToneRow
        """
        return self._transpose_multiply(0, self._mod - self._default_m)

    @property
    def RM(self):
        """Returns the retrograde of the M of the ToneRow."""
        return self.R.M

    @property
    def RMI(self):
        """Returns the retrograde of the MI of the ToneRow."""
        return self.R.MI

    def ordered(self, order):
        """Overridden so that ToneRows are always ordered."""
        self._ordered = True

    def multiset(self, multi):
        """Overridden so that ToneRows are never multisets."""
        self._multiset = False


class PPCSetBase(SetRowBase):
    """Base class for PCSet and PSet"""

    _canon_t = True
    _canon_i = True
    _canon_m = False

    def __sub__(self, other):
        """Remove all instances of a given pc from a pcset"""
        rm_pcs = other
        if isinstance(other, (int, long)):
            rm_pcs = [other]
        if isinstance(other, set):
            rm_pcs = [int(num) for num in other]
        results = self.pitches[:]
        for pc in rm_pcs:
            while pc in results:
                results.remove(pc)
        return self.copy(results)

    def insert(self, place, pitch):
        """
        Given arguments (place, pitch) insert the pitch at the place position.
        Take care to inspect the object's pitches attribute rather than it's
        __repr__, which uses the ppc attribute and may truncate duplicates.
        If the position is too great, the pitch will be appended at the end.
        """
        try:
            pitches = self.pitches[:place]
            pitches.append(pitch)
            pitches.extend(self.pitches[place:])
            self[:] = pitches
        except IndexError:
            self.pitches.append(pitch)

    def clear(self):
        """Remove all pitches/pitch classes from the object."""
        self[:] = []

    def canon(self, t, i, m):
        """
        Takes arguments in the form of (T, I, M) where each is a boolean.
        These arguments determine which TTO's are canonical. These TTO's are
        used to determine an object's set-class.
        (The default canonical operators are T and I, hence the common name
        Tn/TnI type).
        Ex:
            a.canon(True, False, False)

            a.prime would now give the Tn-type, and ignore inversion as an
            operation for determining set-class membership.
        """
        self._canon_t = True if t else False
        self._canon_i = True if i else False
        self._canon_m = True if m else False

    @property
    def get_canon(self):
        """
        Returns a three tuple showing which TTO's are canonical for the given
        object. These are in the order (T, I, M). Refer to canon() for details
        on how these settings are used.
        """
        return (self._canon_t, self._canon_i, self._canon_m)

    @staticmethod
    def fromint(integer, modulus=12):
        """
        Static method that returns a PCSet object with pc's generated from
        their integer representation.
            Ex:
                0 = [], 1 = [0], 2 = [1], 3 = [0, 1], 4 = [2], 5 = [0, 2]
                PCSet.fromint(5) returns PCSet([0, 2])
        """
        new_set = PCSet(mod=modulus)
        new_set.pitches = utils.fromint(integer)
        return new_set

    @property
    def setint(self):
        """
        Returns the integer representation for the unique PC's in a given
        object
        """
        return utils.setint(self._unique_pcs)

    @property
    def pcint(self):
        """
        Returns the integer representation of a given object in prime form.
        """
        return utils.setint(self.prime)

    def each_set(self):
        """
        Yields every possible set in the modulus of the given object.
        """
        return self.each_set_in_mod(self._mod)

    def each_prime(self):
        """
        Yields each unique set-class in the modulus of the given object.
        """
        return self.each_prime_in_mod(self._mod)

    def each_card(self):
        """
        Yields every set with the same cardinality as the given object, taking
        into account the object's modulus.
        """
        return self.each_card_in_mod(self.cardinality, self._mod)

    @classmethod
    def each_set_in_mod(cls, mod):
        return(cls(utils.fromint(integer)) for integer in xrange(0, 2 ** mod))

    @classmethod
    def each_prime_in_mod(cls, mod):
        return(each for each in cls.each_set_in_mod(mod) if each.prime == each)

    @classmethod
    def each_card_in_mod(cls, card, mod):
        """
        Same as the instance method but takes two args for cardinality and
        modulus respectively
        """
        return (cls(each) for each in combinations(cls.each_n_in_mod(mod), card))

    @classmethod
    def each_prime_in_card_mod(cls, card, mod):
        """
        Yields every unique prime form with a given cardinality in the given
        modulus
        """
        for each in combinations(cls.each_n_in_mod(mod), card):
            if cls(each).prime == cls(each):
                yield cls(each)

    @property
    def cardinality(self):
        """Returns the cardinality of the given object."""
        return len(self._pc_set)

    # Helper functions for prime()
    def _t_rotations(self):
        """
        Same as the public method, but enforces that the returned objects are
        PCSets for use with the prime method.
        """
        return (PCSet(self.copy(rot)) for rot in [self._transpose(n) \
                                                  for n in self.each_n()])

    def _i_rotations(self):
        """
        Same as the public method, but enforces that the returned objects are
        PCSets for use with the prime method.
        """
        return (PCSet(self.copy(rot)) for rot in [self._invert(n) \
                                                  for n in self.each_n()])

    def _m_rotations(self):
        """
        Same as the public method, but enforces that the returned objects are
        PCSets for use with the prime method.
        """
        return (PCSet(self.copy(rot)) for rot in [self._transpose_multiply(n) \
                                                  for n in self.each_n()])

    def _mi_rotations(self):
        """
        Same as the public method, but enforces that the returned objects are
        PCSets for use with the prime method.
        """
        return (PCSet(self.copy(rot)) for rot in [self._transpose_multiply(n, self._default_m * -1) \
                                                  for n in self.each_n()])

    @property
    def _rotation_ints(self):
        """
        Returns a nested list with the interger representation for each object
        returned from _rotations()
        """
        return [[utils.setint(pcs) for pcs in rotation] \
            for rotation in self._rotations()]

    def _rotations(self):
        """
        Returns a nested list of PCSet objects for each canonical TTO for a
        given object.
        """
        def setify(pitches):
            pcs = [pitch % self._mod for pitch in pitches]
            result = list(set(pitches))
            result.sort()
            return result

        result = []
        if self._canon_t:
            result.append(self._t_rotations())
            if self._canon_i:
                result.append(self._i_rotations())
            if self._canon_m:
                result.append(self._m_rotations())
            if self._canon_i and self._canon_m:
                result.append(self._mi_rotations())
        else:
            result.append([setify(self.pcs)])
            if self._canon_i:
                result.append([setify(self._invert())])
            if self._canon_m:
                result.append([setify(self._transpose_multiply(0))])
        return result

    @property
    def prime_operation(self):
        """
        A property that returns (n, m) to perform on the given object via TnMm
        in order to obtain its prime form.
        """
        low_vals =[min(izip(operation, count())) \
            for operation in self._rotation_ints]
        min_val = min(izip(low_vals, count()))
        n = min_val[0][1]
        oper = min_val[1]
        if oper == 0:
            m = 1
        elif oper == 1:
            m = self._mod - 1
        elif oper == 2:
            m = self._default_m
        elif oper == 3:
            m = self._mod - self._default_m
        return (n, m)

    @property
    def prime(self):
        """
        Return a PCSet that represents the given object in prime form, taking
        into account its canonical TTO's (set these with .canon(T, I, M)).
        """
        n, m = self.prime_operation
        return PCSet(self.copy(utils.transpose_multiply(self.pitches, n, m)))

    @property
    def mpartner(self):
        """
        Return a PCSet for the M-partner of the given object.
        """
        return PCSet(self._transpose_multiply().prime)

    @staticmethod
    def forte_name(fname):
        """
        A static method that returns a PCSet object with the fort-name provided
        as a string argument.
        Returns an empty PCSet if the argument is not a string with a valid
        Forte name.
        """
        fset = utils.from_forte(fname)
        new_set = PCSet()
        if fset:
            new_set.pitches = fset
        return new_set

    @property
    def forte(self):
        """
        Returns the Forte name for the given object.
        """
        return utils.forte_name(self.pcint)

    @property
    def literal_compliment(self):
        """Returns a PCSet of the literal compliment of the given object."""
        return PCSet(self.copy([n for n in self.each_n() if n not in self.pcs]))

    @property
    def abstract_compliment(self):
        """Returns a PCSet of the abstract compliment of the given object."""
        return PCSet(self.literal_compliment.prime)

    @property
    def icv(self):
        """Returns the interval class vector of the given object."""
        return utils.icv(self._unique_pcs, self._mod)

    @property
    def zpartner(self):
        """
        Property that returns the Z-partner of the given object if it exists,
        otherwise returns None.
        """
        if self._mod > 12:
            return None
        if self._mod == 12:
            zint = Z_PARTNERS.get(self.pcint, None)
            if zint:
                return self.copy(utils.fromint(zint))
            else:
                return
        for each in self.each_card():
            if each.icv == self.icv:
                if each.prime._unique_pcs != self.prime._unique_pcs:
                    return self.copy(p)

    @property
    def invariance_vector(self):
        """
        A property that returns the list of (n, m) pairs that produce an
        invariant set via TnMm
        """
        return [(n, m) for n, m in self.each_tto() if self._transpose_multiply(n, m) == self]

    def supersets(self, limit=0):
        """
        Yields the supersets of the given object. Takes an optional argument,
        which limits the supersets to those with a cardinality <= the limit.
        With no argument, returns all supersets.
        """
        for sup in utils.supersets(self.ppc, self._mod, limit):
            yield self.copy(sup)

    def superprimes(self, limit=0):
        """
        Yields the supersets of the given object which have a unique set-class.
        Takes an optional limit argument with the same behavior as supersets()
        """
        for sup in self.supersets(limit):
            yield PCSet(self.copy(utils.fromint(sup.pcint)))

    def subsets(self, limit=0):
        """
        Yields the subsets of the given object. Takes an optional argument,
        which limits the subsets to those with a cardinality >= the limit.
        With no argument, returns all subsets.
        """
        for sub in utils.subsets(self.ppc, limit):
            yield self.copy(sub)

    def subprimes(self, limit=0):
        """
        Yields the subsets of the given object which have a unique set-class.
        Takes an optional limit argument with the same behavior as subsets().
        """
        for sub in self.subsets(limit):
            yield PCSet(self.copy(utils.fromint(sub.pcint)))


class PCSet(PPCSetBase, PCBase):
    """
    A Class for pitch class sets which adds pitch class only methods
    """

    def c(self):
        """Change the given object in place to its literal compliment."""
        self[:] = self.literal_compliment.pitches

    def z(self):
        """
        Change the given object in place to its Z-partner if possible.
        Otherwise leave the object unchanged.        
        """
        other = self.zpartner
        if other:
            self.pitches = other.pitches


class PSet(PPCSetBase):
    """A class for pitch sets, which adds pitch set only methods."""
    pass


class InvalidTTO(Exception):
    pass


def transpose(a, n):
    return a.copy(utils.transpose(a.pitches, n))

def invert(a, n=0):
    return a.copy(utils.invert(a.pitches, n))

def multiply(a, m=5):
    if a.__class__ == PSet:
        raise InvalidTTO('Pitch sets can not be operated on with TnMm')
    return a.copy(utils.multiply(a.pcs, m))

def transpose_multiply(a, n, m=5):
    if a.__class__ == PSet:
        raise InvalidTTO('Pitch sets can not be operated on with TnMm')
    return a.copy(utils.transpose_multiply(a.pcs, n, m))



#!/usr/bin/env python
from unittest import TestCase, main

from core import PPCSetBase, PCSet, PSet
import utils


"""Test PCSet/PSet class overrides"""
class InitTests(TestCase):
    """PCSet and PSet classes initialize correctly given diff. object types"""

    def setUp(self):
        self.l = [3, 0, 9]
        self.l2 = [2, 13, -12]

    def testInitInt(self):
        a = PSet(0, 9)
        b = PCSet(self.l2)        
        self.assertEqual(a, [0, 9])
        self.assertEqual(b._unique_pcs, [0, 1, 2])

    def testInitSet(self):
        s1 = set(['4', 5, '0'])
        s2 = set(self.l)
        a = PSet(s1)
        b = PCSet(s2)
        c = PCSet(s1, s2)
        self.assertEqual(a._unique_pcs, [0, 4, 5])
        self.assertEqual(b._unique_pcs, [0, 3, 9])
        self.assertEqual(c._unique_pcs, [0, 3, 4, 5, 9])

    def testInitList(self):
        a = PSet(self.l)
        b = PCSet(self.l2)
        c = PSet(self.l, self.l2)
        self.assertEqual(a._unique_pcs, [0, 3, 9])
        self.assertEqual(b._unique_pcs, [0, 1, 2])
        self.assertEqual(c._unique_pcs, [0, 1, 2, 3, 9])
    
    def testInitTuple(self):
        a = PSet((0, 1))
        self.assertEqual(a, [0, 1])

    def testInitPCSet(self):
        a = PCSet(self.l)
        b = PSet(self.l2)
        c = PCSet(a)
        d = PCSet(b)
        e = PCSet(c, d)
        self.assertEqual(c._unique_pcs, [0, 3, 9])
        self.assertEqual(d._unique_pcs, [0, 1, 2])
        self.assertEqual(e._unique_pcs, [0, 1, 2, 3, 9])

    def testInitPSet(self):
        a = PSet([0, 15, 6])
        b = PCSet([0, 1, 9])
        c = PSet(a)
        d = PSet(b)
        e = PSet([2, 8], multiset=True, ordered=False)
        e += c + d
        f = PSet([2, 8] + c + d, multiset=True, ordered=False)
        self.assertEqual(c, [0, 15, 6])
        self.assertEqual(d, [0, 1, 9])
        self.assertTrue(e == [0, 15, 6, 0, 1, 9, 2, 8])
        self.assertTrue(f == [0, 15, 6, 0, 1, 9, 2, 8])


class AddTests(TestCase):
    """Adding to PCSet and PSet objects returns expected objects"""

    def setUp(self):
        self.i = 2
        self.l = [0, 3, 6, 9]
        self.s = set(self.l)

    def testaddInt(self):
        a = PPCSetBase() + self.i
        self.assertEqual(a, self.i)

    def testaddList(self):
        a = PSet() + self.l
        self.assertEqual(a, self.l)

    def testaddTuple(self):
        a = PSet()
        a += (0, 1, 4)
        self.assertEqual(a, (0, 1, 4))

    def testaddSet(self):
        a = PCSet() + self.s
        self.assertEqual(a, self.s)

    def testaddPPCSetTypes(self):
        a = PPCSetBase([0, 1])
        b = PPCSetBase([2, 3])
        c = PPCSetBase()
        c = a + b
        self.assertEqual(c, PSet([0, 1, 2, 3]))
        self.assertEqual(c, PCSet([0, 1, 2, 3]))


class SubTests(TestCase):
    """subtraction override for removes all PC's in PCSet"""

    def setUp(self):
        self.l = [6, 5, 0, 1, 2, 3, 2, 6, 2, 4, 4, 5, 6]

    def testsubInt(self):
        a = PCSet(self.l) - 2
        self.assertEqual(a._unique_pcs, [0, 1, 3, 4, 5, 6])

    def testsubList(self):
        a = PCSet(self.l) - [0, 1, 2]
        self.assertEqual(a._unique_pcs, [3, 4, 5, 6])

    def testsubTuple(self):
        a = PCSet(self.l) - (0, 1, 2, 3, 4)
        self.assertEqual(a._unique_pcs, [5, 6])

    def testsubSet(self):
        a = PCSet(self.l) - set([4, 5, 6])  
        self.assertEqual(a._unique_pcs, [0, 1, 2, 3])

    def testsubPSet(self):
        a = PCSet(self.l) - PSet(0, 1, 5, 6)
        self.assertEqual(a._unique_pcs, [2, 3, 4])

    def testsubPCSet(self):
        a = PCSet(self.l) - PCSet(0, 1, 2, 3)
        self.assertEqual(a._unique_pcs, [4, 5, 6])


class EqualityTest(TestCase):
    """Equality checks between PCSets, PSets, and built ins"""

    def setUp(self):
        self.l = [0, 1, 2, 4, 5, 10 ,1 ,0]
        self.s = set(self.l)
        self.pcset = PCSet(self.l)
        self.pset = PSet(self.l)
        self.pset.ordered(False)

    def testeqPPCSets(self):
        self.assertTrue(self.pcset == self.pcset)
        self.assertTrue(self.pset == self.pset)
        self.assertTrue(self.pcset == self.pset)

    def testeqPPCSet_list(self):
        self.assertTrue(self.pcset == self.l and self.pset == self.l)

    def testeqPPCSet_set(self):
        self.assertTrue(self.pcset == self.s and self.pset == self.s)

    def testeqPPCSet_int(self):
        a = PCSet(3)
        b = PSet(3)
        self.assertTrue(a == 3 and b == 3)


class InEqualityTest(TestCase):
    """Check for != between PCSets, PSets, and built ins"""

    def setUp(self):
        self.l = [0, 1, 2, 4, 5, 10 ,1 ,0]
        self.l2 = [0, 1, 2, 4, 5, 10]
        self.l3 = [0, 2, 4]
        self.s = set(self.l)
        self.s2 = set(self.l2)
        self.pcset = PCSet(self.l)
        self.pset = PSet(self.l)
        self.pcset2 = PCSet(self.l2)
        self.pset2 = PSet(self.l2)

    def testnePPCSets(self):
        pset3 = PSet(self.l3)
        self.assertTrue(self.pcset == self.l2 and self.pset == self.l2)
        self.assertTrue(self.pcset == self.pset and self.pcset != pset3)

    def testnePPCSet_list(self):
        self.assertTrue(self.pcset == self.l2 and self.pset == self.l2)
        self.assertTrue(self.pcset != self.l3 and self.pset != self.l3)

    def testnePPCSet_set(self):
        s = set([0, 1, 2])
        self.assertTrue(self.pcset != s and self.pset != s)

    def testnePPCSet_int(self):
        self.assertTrue(self.pcset != 5 and self.pset != 3)


class GetItemTest(TestCase):
    """[x:y] PCSets and PSets should return pcs and pitches respectively"""
    
    def setUp(self):
        self.l = [0, 1, 2, 5, 7, 9, 35, 4]
        self.pcset = PCSet(self.l)
        self.pset = PSet(self.l)

    def testgetInt(self):
        self.assertEqual(self.pcset[6], [9])
        self.assertEqual(self.pset[6], [9])
        self.pset.ordered(True)
        self.assertEqual(self.pset[6], [35])

    def testgetRange(self):
        self.assertEqual(self.pcset[2:5], [2, 4, 5])

    def testgetNegative(self):
        self.assertEqual(self.pcset[-1], [11])

    def testgetAfter(self):
        try:
            self.pcset[100]
        except IndexError:
            assert True


class SequenceTest(TestCase):
    """PCSets and PSets behave like sequences with iteration and len()"""

    def setUp(self):
        self.l = [0, 1, 2, 4]
        self.pcset = PCSet(self.l)
        self.pset = PSet(self.l)

    def testiterTest(self):
        pcout = [pc for pc in self.pcset]
        pout = [p for p in self.pset]
        self.assertTrue(pcout == self.l and pout == self.l)

    def testlenTest(self):
        self.assertEqual(len(self.pcset), 4)
        self.pset.multiset(False)
        self.assertEqual(len(self.pset + [0, 2, 5]), 5)
        self.pset.multiset(True)
        self.assertEqual(len(self.pset + [0, 2, 5]), 7)


class ReprTest(TestCase):
    """PCSet and PSet repr's vary based on ordered and multiset settings"""

    def setUp(self):
        self.l = [0, 1, -7, 16, 1, 0]
        self.pcset = PCSet(self.l)
        self.pset = PSet(self.l)

    def testreprO(self):
        self.pcset.ordered(True)
        self.pset.ordered(True)
        self.pcset.multiset(False)
        self.pset.multiset(False)
        self.assertEqual(self.pcset.__repr__(), '[0, 1, 5, 4]')
        self.assertEqual(self.pset.__repr__(), '[0, 1, -7, 16]')

    def testreprO_Multi(self):
        pcset = PCSet(self.l, ordered=True, multiset=True)
        pset = PSet(self.l, ordered=True, multiset=True)
        self.assertEqual(pcset.__repr__(), '[0, 1, 5, 4, 1, 0]')
        self.assertEqual(pset.__repr__(), '[0, 1, -7, 16, 1, 0]')

    def testreprUO(self):        
        self.pcset.ordered(False)
        self.pset.ordered(False)
        self.pcset.multiset(False)
        self.pset.multiset(False)
        self.assertEqual(self.pcset.__repr__(), '[0, 1, 4, 5]')
        self.assertEqual(self.pset.__repr__(), '[-7, 0, 1, 16]')

    def testreprUO_Multi(self):
        pcset = PCSet(self.l, ordered=False, multiset=True)
        pset = PSet(self.l, ordered=False, multiset=True)
        self.assertEqual(pcset.__repr__(), '[0, 0, 1, 1, 4, 5]')
        self.assertEqual(pset.__repr__(), '[-7, 0, 0, 1, 1, 16]')


"""Test methods pitch/pc manipulation methods such as insert, copy, clear"""
class InsertTest(TestCase):
    """
    Insert method inserts a pitch/pc in place or appends it for index errors
    """

    def setUp(self):
        self.l = [1, 4, 23, 5, -9]

    def testinsertBegin(self):
        a = PCSet(self.l)
        b = PSet(self.l)
        a.insert(0, 10)
        b.insert(0, 11)
        self.assertEqual(a.pitches, [10] + self.l)
        self.assertEqual(b.pitches, [11] + self.l)

    def testinsertMiddle(self):
        a = PCSet(self.l)
        b = PSet(self.l)
        a.insert(3, 15)
        b.insert(4, 8)
        self.assertEqual(a.pitches, [1, 4, 23, 15, 5, -9])
        self.assertEqual(b.pitches, [1, 4, 23, 5, 8, -9])

    def testinsertEnd(self):
        a = PCSet(self.l)
        b = PSet(self.l)
        a.insert(6, 10)
        b.insert(6, 99)
        self.assertEqual(a.pitches, self.l + [10])
        self.assertEqual(b.pitches, self.l + [99])

    def testinsertAfter(self):
        a = PCSet(self.l)
        b = PSet(self.l)
        a.insert(100, 1)
        b.insert(100, 0)
        self.assertEqual(a.pitches, self.l + [1])
        self.assertEqual(b.pitches, self.l + [0])

    def testinsertNegative(self):
        a = PCSet(self.l, multiset=True)
        b = PSet(self.l, multiset=True)
        a.insert(-2, 2)
        b.insert(-3, 12)
        self.assertEqual(a.pitches, [1, 4, 23, 2, 5, -9])
        self.assertEqual(b.pitches, [1, 4, 12, 23, 5, -9])


class ClearTest(TestCase):
    """The clear method removes all pitches/pcs from a PCSet or PSet"""

    def setUp(self):
        self.l = [9, 3, 4, 5, 6, 1]

    def testclearPCSet(self):
        a = PCSet(self.l)
        a.clear()
        self.assertTrue(a.pcs == [] and a.pitches == [])

    def testclearPSet(self):
        a = PSet(self.l)
        a.clear()
        self.assertTrue(a.pcs == [] and a.pitches == [])


class CopyTest(TestCase):
    """
    Create a new instance with the pitches, settings and class of the original
    """

    def setUp(self):
        self.l = [0, 3, 16, 1]

    def testcopyPitches(self):
        a = PCSet(self.l)
        b = PSet(self.l)
        c = a.copy()
        d = b.copy()
        self.assertTrue(c.pitches == self.l and d.pitches == self.l)

    def testcopySettings(self):
        def verify(current):
            self.assertEqual(current.__class__, PCSet)
            self.assertTrue(current._ordered == False and current._multiset == False)
            self.assertEqual(current._mod, 7)
            self.assertTrue(current._canon_t, True)
            self.assertEqual(current._canon_i, False)
            self.assertEqual(current._canon_m, True)
            self.assertEqual(current.pitches, self.l)

        a = PCSet(self.l)
        a.mod(7)
        a.ordered(False)
        a.multiset(False)
        a.canon(True, False, True)
        a2 = a.copy()
        verify(a2)    
        b = PSet(self.l)
        self.assertEqual(b.__class__, PSet)
        b = a.copy()
        verify(b)


"""Test settings attributes and their methods"""
class SettingsTest(TestCase):
    """PCSet and PSet instances have various methods to change settings"""

    def setUp(self):
        self.l = [0, 4, 9, 11]
        self.pcset = PCSet(self.l)
        self.pset = PSet(self.l)

    def testMod(self):
        self.pcset.mod(7)
        self.pset.mod(13)
        self.assertEqual(self.pcset._mod, 7)
        self.assertEqual(self.pset._mod, 13)

    def testCanon(self):
        a = self.pcset
        b = self.pset
        a.canon(True, False, False)
        b.canon(False, True, True)        
        self.assertTrue(
            a._canon_t == True and a._canon_i == False and a._canon_m == False
        )
        self.assertTrue(
            b._canon_t == False and b._canon_i == True and b._canon_m == True
        )

    def testDefault_m(self):
        self.pcset.default_m(7)
        self.assertEqual(self.pcset._default_m, 7)

    def testOrdered(self):
        self.pcset.ordered(True)
        self.pset.ordered(True)
        self.assertTrue(
            self.pcset._ordered == True and self.pset._ordered == True
        )
        self.pcset.ordered(False)
        self.pset.ordered(False)
        self.assertTrue(
            self.pcset._ordered == False and self.pset._ordered == False
        )

    def testMultiset(self):
        self.pcset.multiset(True)
        self.pset.multiset(True)
        self.assertTrue(
            self.pcset._multiset == True and self.pset._multiset == True
        )
        self.pcset.multiset(False)
        self.pset.multiset(False)
        self.assertTrue(
            self.pcset._multiset == False and self.pset._multiset == False
        )



#!/usr/bin/env python
from unittest import TestCase, main

from core import ToneRow


class ToneRowTests(TestCase):        
    def setUp(self):
        self.l = [0, 4, 5, 8, 9, 1, 10, 3, 6, 2, 7, 11]
        self.tonerow = ToneRow(self.l)

    def testInit(self):
        # Make sure exception is thrown if args < mod
        try:
            tonerow = ToneRow([0])
        except ToneRow.IncompleteToneRow:
            self.assertTrue(True)
        else:
            self.assertTrue(False)
        valid_tr = ToneRow(0, 1, 2, mod=3)
        self.assertTrue(valid_tr)

    def testMultiset_fails(self):
        self.tonerow.multiset(True)
        self.assertEqual(self.tonerow._multiset, False)

    def testP(self):
        self.assertEqual(self.tonerow.P, self.l)

    def testR(self):
        r = self.l
        r.reverse()
        self.assertEqual(self.tonerow.R, r)

    def testI(self):
        i = self.tonerow._invert()
        self.assertEqual(self.tonerow.I, i)

    def testRI(self):
        i = self.tonerow._invert()
        ri = i.ppc
        ri.reverse()
        self.assertEqual(self.tonerow.RI, ri)       



#!/usr/bin/env python
from unittest import TestCase, main

from core import PCSet, PSet
import utils


"""Test pc and set methods"""
class PCSetifyTests(TestCase):
    """Test pcs, _pc_sets, _setify, _unique_pcs and related methods"""

    def setUp(self):        
        self.pcset = PCSet(22, 3, 5, -7, 0, 10)

    def testPc(self):
        pcs = [pitch % self.pcset._mod for pitch in self.pcset.pitches]
        self.assertEqual(self.pcset.pcs, pcs)

    def testPc_set(self):
        self.assertEqual(self.pcset._pc_set, set([0, 3, 5, 10]))

    def testCardinality(self):
        self.assertEqual(self.pcset.cardinality, 4)

    def testPitch_set(self):
        self.assertEqual(self.pcset._pitch_set, set([22, 3, 5, -7, 0, 10]))

    def testUo_pcs(self):
        self.assertEqual(self.pcset.uo_pcs, [0, 3, 5, 5, 10, 10])

    def testUnique_pcs(self):
        self.assertEqual(self.pcset._unique_pcs, [0, 3, 5, 10])


"""Test Tn TnI TnMm operations"""
class VarEqualsTnTm(TestCase):
    """operations that return a PCSet or PSet"""

    def setUp(self):
        self.l = [0, 2, 5, 20, -1]
        self.pcset = PCSet(self.l)
        self.pset = PSet(self.l)

    def testTranspose(self):
        a = self.pcset._transpose(5)
        self.assertEqual(a.pitches, [5, 7, 10, 25, 4])
        self.assertEqual(a.uo_pcs, [1, 4, 5, 7, 10])
        self.assertEqual(a.__class__, PCSet)
        b = self.pset._transpose(5)
        self.assertEqual(b.pitches, [5, 7, 10, 25, 4])
        self.assertEqual(b.__class__, PSet)

    def testInvert(self):
        a = self.pcset._invert()
        self.assertEqual(a.pitches, [0, -2, -5, -20, 1])
        self.assertEqual(a.uo_pcs, [0, 1, 4, 7, 10])
        self.assertEqual(a.__class__, PCSet)
        b = self.pset._invert(3)
        self.assertEqual(b.pitches, [3, 1, -2, -17, 4])
        self.assertEqual(b.__class__, PSet)

    def testTranspose_multiply(self):
        a = self.pcset._transpose_multiply(1, 11)
        b = self.pset._transpose_multiply(5, 5)
        c = self.pset._transpose_multiply(0, 5)
        d = self.pcset._transpose_multiply(1, 1)
        self.assertEqual(a, a._transpose_multiply(0, 1))
        self.assertEqual(a.pcs, [1, 11, 8, 5, 2])
        self.assertEqual(b.pcs, [5, 3, 6, 9, 0])
        self.assertEqual(c.pcs, self.pcset._transpose_multiply())
        self.assertEqual(d, self.pcset._transpose(1))


class TnTmInPlace(TestCase):
    """operations that change a PCSet/PSet in place via TnTm"""

    def setUp(self):
        self.l = [0, -15, 20, 4, 2, 0]
        self.a = PCSet(self.l, multiset=True)
        self.b = self.a.copy()
        self.c = self.a.copy()

    def testT(self):
        a = self.a
        b = self.b
        a.t(5)
        b.t(-2)
        self.assertEqual(a.pitches, [5, -10, 25, 9, 7, 5])
        self.assertEqual(b.pitches, [-2, -17, 18, 2, 0, -2])

    def testI(self):
        a = self.a
        b = self.b
        c = self.c
        a.i()
        b.i()
        c.i(3)
        self.assertEqual(a.pitches, [0, 15, -20, -4, -2, 0])
        self.assertEqual(a, b.pcs)
        self.assertEqual(b.pitches, a.pitches)
        self.assertEqual(c.pitches, [3, 18, -17, -1, 1, 3])

    def testM(self):
        a = self.a
        b = self.b
        a.m()
        b.m(3)
        self.assertEqual(a._unique_pcs, [0, 4, 8, 9, 10])
        self.assertEqual(b._unique_pcs, [0, 1, 3, 7, 11])

    def testMi(self):
        a = self.a
        b = self.b
        c = self.c
        a.mi()
        b.m()
        b.i()
        c.mi(5)
        self.assertEqual(a._unique_pcs, [0, 2, 3, 4, 8])
        self.assertEqual(a, b)
        self.assertEqual(c._unique_pcs, [1, 5, 7, 8, 9])

    def testTm(self):
        a = self.a
        b = a.copy()
        c = b.copy()
        a.t_m(0, 1)
        b.t_m(1, 5)
        c.t_m(5, -1)
        self.assertEqual(a._unique_pcs, [0, 2, 4, 8, 9])
        self.assertEqual(b._unique_pcs, [1, 5, 9, 10, 11])
        self.assertEqual(c._unique_pcs, [1, 3, 5, 8, 9])


class BinIntTest(TestCase):
    """Functions that convert PCSets to integers and vice versa"""

    def setUp(self):
        self.pcset = PCSet(1, 4)
        self.pcset2 = PCSet(0, 3, 4, 6)

    def testSetInt(self):
        self.assertEqual(self.pcset.setint, 18)
        self.assertEqual(self.pcset2.setint, 89)

    def testFromInt(self):
        self.assertEqual(utils.fromint(18), self.pcset._unique_pcs)
        self.assertEqual(utils.fromint(89), self.pcset2._unique_pcs)

    def testNewFromInt(self):
        """Using fromint staticmethod to create new PCSets"""
        a = PCSet.fromint(5)
        self.assertEqual(a, [0, 2])
        b = PCSet.fromint(630, 7)
        self.assertEqual(b, [1, 2, 4, 5, 6])


class EachTest(TestCase):
    """Methods that provide each n in the modulus or each set in the modulus"""

    def setUp(self):
        self.pcset = PCSet()

    def testEach_n(self):
        self.pcset.mod(12)
        self.assertEqual(list(self.pcset.each_n()),
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11])
        self.pcset.mod(6)
        self.assertEqual(list(self.pcset.each_n()),[0, 1, 2, 3, 4, 5])

    def testEach_set(self):
        self.pcset.mod(12)
        each_set = list(self.pcset.each_set())
        self.assertEqual(each_set[0:6], [[], [0], [1], [0, 1], [2], [0, 2]])
        self.assertEqual(each_set[-1],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11])
        for index, each in enumerate(each_set):
            self.assertEqual(utils.setint(each), index)
        self.pcset.mod(4)
        each_set = list(self.pcset.each_set())
        self.assertEqual(each_set[0:3], [[], [0], [1]])
        self.assertEqual(each_set[-1], [0, 1, 2, 3])



#!/usr/bin/env python
from const import FORTE_NAMES, FORTE_INTS

def transpose(pitches, sub_n=0):
    """
    Given a list of pitches and n, returns an object of the same type after Tn
    """
    return [pitch + sub_n for pitch in pitches]

def invert(pitches, sub_n=0):
    """
    Given an object and n, returns an object of the same type after TnI.
    If n is not provided, n is assumed to be 0
    """
    return transpose(multiply(pitches, -1), sub_n)

def multiply(pitches, sub_m):
    """
    Given an object and n, returns an object of the same type after TnMm,
    where m is required. (For mod 12, m is usually 5.)
    """
    return [pitch * sub_m for pitch in pitches]

def transpose_multiply(pitches, sub_n, sub_m):
    """
    Given an object, n, and m, returns an object of the same type after TnMm.
    All arguments are required.
    """
    result = multiply(pitches, sub_m)
    return transpose(result, sub_n)

def setint(pcs):
    """Find the integer representation of an unordered PC set"""
    return sum([2 ** pc for pc in pcs])

def fromint(integer):
        result = []
        limit = len(bin(integer)) - 2
        each_digit = [2 ** n for n in xrange(limit, -1, -1)]
        for index, digit in enumerate(each_digit):
            if integer >= digit:
                integer -= digit
                result.append(limit - index)
        result.sort()
        return result

def forte_name(setint):
	return FORTE_NAMES.get(setint)

def forte_int(fname):
    return FORTE_INTS.get(fname)       

def from_forte(fname):
    try:
        return fromint(forte_int(fname))
    except:
        return None

def icv(pcs, mod=12):
    icv_length = (mod / 2) + 1;
    result = [0 for num in xrange(0, icv_length)]
    icvs = xrange(0, icv_length)
    for pc in pcs:
        for cv in icvs:
            if (pc + cv) % mod in pcs:
                result[cv] += 1
    if icv_length - 1 == mod / 2.0:
        result[icv_length - 1] = result[icv_length - 1] / 2
    return result

def _supers_n_plus_1(pcs, mod):
    for index in xrange(0, mod):
        if index not in pcs:
            result = pcs[:]
            result.append(index)
            yield result

def _subs_n_minus_1(pcs):
    for index in xrange(0, len(pcs)):
        yield [pc for i, pc in enumerate(pcs) if i != index]

def supersets(pcs, mod, limit=0):
    if not limit:
        limit = mod
    # FIXME: Raise a useful exception rather than just yielding None
    if limit <= len(pcs):
        yield
    else:
        for sup in _supers_n_plus_1(pcs, mod):
            current = sup
            yield current
            if len(current) < mod and len(current) < limit:
                for pcs in supersets(current, mod, limit):
                    yield pcs

def subsets(pcs, limit=0):
    # FIXME: Raise a useful exception rather than just yielding None
    if limit >= len(pcs):
        yield
    else:
        for sub in _subs_n_minus_1(pcs):
            current = sub
            yield current
            if len(current) > limit:
                for pcs in subsets(current, limit):
                    yield pcs



#!/usr/bin/env python
from unittest import TestCase, main

from core import PCSet, PSet
import utils
from const import Z_PARTNERS

class PrimeTestCase(TestCase):        
    def setUp(self):
        self.l = [0, 1, 2, 3, 4, 6]
        self.set_ints = [0, 4095, 392, 661, 583, 203, 2741, 584, 394, 858]
        self.sets = [PCSet(utils.fromint(each)) for each in self.set_ints]
        self.pcset = PCSet(self.l)
        self.t_rots = list(self.pcset._t_rotations())
        self.i_rots = list(self.pcset._i_rotations())
        self.m_rots = list(self.pcset._m_rotations())
        self.mi_rots = list(self.pcset._mi_rotations())

"""Test prime and related subfunctions"""
class PrimeAndRotationsTest(PrimeTestCase):
    """Methods that generate lists of PCSets after TnTm operations"""

    def make_rotations(self):
        """Helper for checking ._rotations and ._rotation_ints"""
        self.t_only = [[self.pcset.uo_pcs]]
        self.m_only = self.t_only + [[self.pcset._transpose_multiply().uo_pcs]]
        self.i_only = self.t_only + [[self.pcset._invert().uo_pcs]]
        self.im_only = self.t_only + [[self.pcset._invert().uo_pcs]] + \
            [[self.pcset._transpose_multiply().uo_pcs]]
        self.ti_rots = [self.t_rots] + [self.i_rots]
        self.tm_rots = [self.t_rots] + [self.m_rots]
        self.im_rots = [self.i_rots] + [self.m_rots]
        self.all_rots = [self.t_rots] + [self.i_rots] + [self.m_rots] + \
            [self.mi_rots]

    def make_canons(self):    
        for canon_t in [True, False]:
            for canon_i in [True, False]:
                for canon_m in [True, False]:
                    yield canon_t, canon_i, canon_m

    def make_rotation_ints(self):
        """Helper for making setints from rotations"""
        result = []
        for rotation in self.pcset._rotations():
            result.append([utils.setint(pcs) for pcs in rotation])
        return result

    def make_prime_operation(self):
        """Helper for listing operations that make rotation_ints"""
        tto_ints = []
        for index_m, operation in enumerate(self.pcset._rotation_ints):
            for index_t, num in enumerate(operation):
                tto_ints.append((num, index_t, index_m))
        tto_ints.sort()
        tto = tto_ints[0]
        oper = tto[2]
        if oper == 0:
            m = 1
        elif oper == 1:
            m = self.pcset._mod - 1
        elif oper == 2:
            m = self.pcset._default_m
        elif oper == 3:
            m = self.pcset._mod - self.pcset._default_m
        return (tto[1], m)


    def testT_rotations(self):
        self.assertEqual(self.t_rots[0], self.pcset.uo_pcs)
        self.assertEqual(self.t_rots[-1], [0, 1, 2, 3, 5, 11])

    def testI_rotations(self):
        self.assertEqual(self.i_rots[0], self.pcset._invert().uo_pcs)
        self.assertEqual(self.i_rots[-1], [5, 7, 8, 9, 10, 11])

    def testM_rotations(self):
        self.assertEqual(self.m_rots[0], self.pcset._transpose_multiply().uo_pcs)
        self.assertEqual(self.m_rots[-1], [2, 4, 5, 7, 9, 11])

    def testMi_rotations(self):
        self.assertEqual(self.mi_rots[0],
                         self.pcset._transpose_multiply()._invert().uo_pcs)
        self.assertEqual(self.mi_rots[-1], [1, 3, 5, 6, 8, 11])

    def testAll_rotations(self):
        all_rots = self.t_rots + self.i_rots + self.m_rots + self.mi_rots
        self.assertEqual(all_rots, self.pcset.all_rotations)

    def testRotations(self):
        self.make_rotations()
        def unpack_rotations():
            """unpack the generator of generators to a list of lists"""
            return [list(rot) for rot in self.pcset._rotations()]
        self.pcset.canon(True, True, True)
        self.assertEqual(self.all_rots, unpack_rotations())
        self.pcset.canon(True, True, False) # Default setting. Tn/TnI-type
        self.assertEqual(self.ti_rots, unpack_rotations())
        self.pcset.canon(True, False, True)
        self.assertEqual(self.tm_rots, unpack_rotations())
        self.pcset.canon(True, False, False)
        self.assertEqual([self.t_rots], unpack_rotations())
        self.pcset.canon(False, True, True)
        self.assertEqual(self.im_only, unpack_rotations())
        self.pcset.canon(False, True, False)
        self.assertEqual(self.i_only, unpack_rotations())
        self.pcset.canon(False, False, True)
        self.assertEqual(self.m_only, unpack_rotations())
        self.pcset.canon(False, False, False)
        self.assertEqual(self.t_only, unpack_rotations())

    def testAllRotations(self):
        all_rots = []
        all_rots.extend(self.t_rots)
        all_rots.extend(self.i_rots)
        all_rots.extend(self.m_rots)
        all_rots.extend(self.mi_rots)
        self.assertEqual(all_rots, self.pcset.all_rotations)

    def testRotationInts(self):
        self.make_rotations()
        for t, i, m in self.make_canons():
            self.pcset.canon(t, i, m)
            self.assertEqual(self.make_rotation_ints(),
                             self.pcset._rotation_ints)

    def testPrimeOperation(self):
        self.pcset.clear()
        l = [3, 5, 6, 11, 0, -2, 2, -4, 7, 9, 13, -8]
        for pc in l:
            self.pcset + pc
            for t, i, m in self.make_canons():
                self.pcset.canon(t, i, m)
                self.assertEqual(self.pcset.prime_operation,
                                 self.make_prime_operation())

    def testPrime(self):
        primes = []
        set_ints = [0, 4095, 420, 2224]
        sets = [PCSet(utils.fromint(each)) for each in set_ints]
        for t, i, m in self.make_canons():
            for each in sets:
                each.canon(t, i, m)
                primes.append(each.prime)
        self.assertEqual(primes, [
                         [],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
                         [0, 1, 3, 6],
                         [0, 1, 4, 6],
                         [],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
                         [0, 1, 3, 6],
                         [0, 1, 3, 7],
                         [],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
                         [0, 6, 7, 9],
                         [0, 1, 6, 10],
                         [],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
                         [0, 3, 5, 6],
                         [0, 1, 3, 7],
                         [],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
                         [2, 5, 7, 8],
                         [1, 5, 7, 8],
                         [],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
                         [2, 5, 7, 8],
                         [1, 5, 7, 8],
                         [],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
                         [2, 5, 7, 8],
                         [4, 5, 7, 11],
                         [],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
                         [2, 5, 7, 8],
                         [4, 5, 7, 11]                            
        ])


class PrimeReliantTests(PrimeTestCase):

    def testPCInt(self):
        for each in self.sets:
            self.assertEqual(utils.setint(each.prime), each.pcint)

    def testForteName(self):
        for each in self.sets:
            self.assertTrue(each.forte != None)

    def testForteInt(self):
        for each in self.sets:
            self.assertEqual(each.pcint,
                             utils.setint(utils.from_forte(each.forte)))

    def testForteSet(self):
        a = PCSet()
        fnames = [each.forte for each in self.sets]
        new_sets = []
        for fname in fnames:
            a.clear()
            a = PCSet.forte_name(fname)
            new_sets.append(a.copy())
        new_fnames = [each.forte for each in new_sets]
        self.assertEqual(fnames, new_fnames)

    def testMPartner(self):
        for each in self.sets:
            self.assertEqual(each.mpartner, each._transpose_multiply().prime)

    def testLiteralCompliment(self):
        self.assertEqual(self.pcset.literal_compliment, [5, 7, 8, 9, 10, 11])

    def testCompliment(self):
        for each in self.sets:
            other = each.copy()
            other.c()
            self.assertEqual(each.literal_compliment, other)

    def testAbstractCompliment(self):
        for each in self.sets:
            self.assertEqual(each.literal_compliment.prime,
                             each.abstract_compliment)

    def testicv(self):
        icvs = [each.icv for each in self.sets]
        self.assertEqual(icvs, [
            [0, 0, 0, 0, 0, 0, 0],
            [12, 12, 12, 12, 12, 12, 6],
            [3, 1, 0, 0, 1, 1, 0],
            [5, 0, 3, 2, 1, 4, 0],
            [5, 2, 1, 2, 2, 2, 1],
            [5, 2, 1, 2, 1, 2, 2],
            [7, 2, 5, 4, 3, 6, 1],
            [3, 0, 0, 2, 0, 0, 1],
            [4, 1, 1, 0, 1, 2, 1],
            [6, 2, 3, 3, 2, 4, 1],
        ])

    def make_z_sets(self):
        zints = Z_PARTNERS.keys()
        return [PCSet(utils.fromint(zint)) for zint in zints]

    def testZPartner(self):
        for each in self.make_z_sets():
            self.assertTrue(each.zpartner.pcint in Z_PARTNERS.values())

    def testZ(self):
        for each in self.make_z_sets():
            each.z()
            self.assertTrue(each.pcint in Z_PARTNERS.values())

    def testEachPrime(self):
        a = PCSet()
        for prime in a.each_prime():
            self.assertEqual(prime.prime, prime._unique_pcs)

class SubsetsTest(TestCase):

    def setUp(self):
        self.l = [0, 3, 4]
        self.pcset = PCSet(self.l)

    def testSubsets(self):
        subs = [sub for sub in self.pcset.subsets()]
        self.assertEqual(subs, [
                         [3, 4], [4], [], [3], [],
                         [0, 4], [4], [], [0], [],
                         [0, 3], [3], [], [0], []
        ])
        self.assertTrue(isinstance(subs[0], PCSet))

    def testSubPrimes(self):
        subprimes = [subprime for subprime in self.pcset.subprimes()]
        self.assertEqual(subprimes, [
                         [0, 1], [0], [], [0], [],
                         [0, 4], [0], [], [0], [],
                         [0, 3], [0], [], [0], []
        ])

class SupersetsTest(TestCase):

    def setUp(self):
        self.l = [0, 1, 2, 3, 5, 6, 8, 10, 11]
        self.pcset = PCSet(self.l)

    def testSupersets(self):
        supers = [sup for sup in self.pcset.supersets()]
        self.assertEqual(supers, [
                         [0, 1, 2, 3, 4, 5, 6, 8, 10, 11],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
                         [0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 11],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
                         [0, 1, 2, 3, 5, 6, 7, 8, 10, 11],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
                         [0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 11],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
                         [0, 1, 2, 3, 5, 6, 8, 9, 10, 11],
                         [0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 11],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
                         [0, 1, 2, 3, 5, 6, 7, 8, 9, 10, 11],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]
        ])

    def testSuperPrimes(self):
        superprimes = [superprime for superprime in self.pcset.superprimes()]
        self.assertEqual(superprimes, [
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 10],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
                         [0, 1, 2, 3, 4, 5, 7, 8, 9, 10],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
                         [0, 1, 2, 3, 4, 5, 6, 7, 9, 10],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]
        ])
"""
TODO: Add tests for PPCSet methods below

    Finish prime test - pass doesn't count as a unit test

Untested methods:
each_card #FIXME: Implementation of this method sucks anyways

Add test for similarity relations

Add tests for Tone rows

"""



#!/usr/bin/env python

import unittest, sys

from test import __all__ as alltests

class NoSuchTestError(Exception): pass

def match(name):
    if name not in alltests:
        raise NoSuchTestError, "Test '%s' not found." % name
    else:
        return name

def run(testnames, level=2):
    for name in testnames:
        if name:
            print "Testing Pyset module: %s" % name
            test = 'test.' + name
            suite = unittest.TestLoader().loadTestsFromName(test)
            unittest.TextTestRunner(verbosity=level).run(suite)

if __name__ == '__main__':
    args = sys.argv[1:]
    if args:
        tests = [match(name) for name in args]
        run(tests)
    else:
        run(alltests,level=1)



#!/usr/bin/env python
from unittest import TestCase, main

from core import PCSet, PSet
from sim import *

class SimMZCTests(TestCase):        
    def setUp(self):
        self.l = [0, 1, 2, 4, 5, 8]
        self.pcset = PCSet(self.l)
        self.pcset_z = PCSet([0, 1, 3, 7])

    def testM(self):
        a = self.pcset
        b = a.mpartner
        self.assertTrue(m(a, b))
        b.t_m(3, -1)
        self.assertTrue(m(a, b))

    def testZ(self):
        a = self.pcset_z
        b = a.zpartner
        self.assertTrue(z(a, b))

    def testC(self):
        a = self.pcset
        b = a.abstract_compliment
        self.assertTrue(c(a, b))

    def testIV(self):
        a = self.pcset
        b = self.pcset_z
        self.assertEqual(iv(a, b), [2, 2, 2, 2, 1, 2, 1, 3, 2, 1, 2, 4])

    def testSIM(self):
        a = self.pcset
        b = self.pcset_z
        self.assertEqual(sim(a, b), 9)

    def testASIM(self):
        a = self.pcset
        b = self.pcset_z
        self.assertEqual(asim(a, b, rational=True), (9, 21))

    def testcheckmod(self):
        a = self.pcset
        b = PCSet(0, 3, mod=7)
        self.assertEqual(iv(a, b), NotImplemented)
        self.assertEqual(sim(a, b), NotImplemented)
        self.assertEqual(asim(a, b), NotImplemented)



from setuptools import setup, find_packages

setup(
    name='sator',
    version=__import__('sator').__version__,
    author='Caleb Smith',
    author_email='caleb.smithnc@gmail.com',
    packages=find_packages(),
    include_package_data=True,
    url='https://github.com/calebsmith/Sator',
    license='GPL',
    zip_safe=True,
    description="Sator is a python module for atonal music theory analysis.",
    classifiers=[
        'Topic :: Other/Nonlisted Topic',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
    ],
    install_requires = [
    ],
)



#!/usr/bin/env python
"""
Module for similarity relations/functions between two pitch or pitch class sets
"""

from __future__ import division


"""Functions for determining if the two sets are m, z, or c partners"""

def z(a, b):
    return a.zpartner == b.prime

def c(a, b):
    return a.abstract_compliment == b.prime

def m(a, b):
    return a.mpartner == b.prime

def zc(a, b):
    return z(a, b) and c(a, b)


""" Similarity Functions """

def _check_mod(a, b):
    # Verify the sets have the same modulus. Most functions are defined if the
    # input sets are different moduli.
    return False if a.mod() == b.mod() else True

"""Robert Morris"""
def iv(a, b):
    """
    How many of each ordered interval is expressed from the PC's of a to those
    of b. -Robert Morris' IV(a, b) as described in
    "Composition with Pitch Classes" 1987
    """
    if _check_mod(a, b):
        return NotImplemented
    ivect = dict([(n, 0) for n in a.each_n()])
    for pc in a.uo_pcs:
        for n in a.each_n():
            lookfor = (pc + n) % a.mod()
            if lookfor in b.pcs:
                ivect[n] += 1
    return ivect.values()

def sim(a, b):
    """
    The sum of differences between the icvs of sets a and b (excluding icv0)
    - Robert Morris SIM(a, b)
    """
    if _check_mod(a, b):
        return NotImplemented
    # Skip the first interval class, which is the cardinality.
    return sum([abs(ic[0] - ic[1]) for ic in zip(a.icv[1:], b.icv[1:])])

def asim(a, b, rational=False):
    if _check_mod(a, b):
        return NotImplemented
    """
    The sum of differences between the icvs of sets a and b (excluding icv0)
    divided by the total icvs in both sets, which is equal to the highest
    possible difference.

    Takes the kwarg rational. If true, returns a rational number in the form of
    a two tuple.
    Defaults to rational=False

    - Robert Morris ASIM(a, b)
    """
    sim = asim = 0
    for ic in zip(a.icv[1:], b.icv[1:]):
        sim += abs(ic[0] - ic[1])
        asim += ic[0] + ic[1]
    return (sim, asim) if rational else sim / asim
